unit filestructure;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Graphics;

Type
  RGun = record
     idname: Cardinal;
     idinfo: Cardinal;
     nickname: string;
     LODranges: string;       // abhängig vom Modell
     DA_archetype: string;
     material_library: string;
     envmap_material: string;
     hp_gun_type: byte;
     HP_child: string;
     hit_pts: Single;
     explosion_resistance: Single;
     explosion_arch: string;
     debris_type: string;
     parent_impulse: Single;
     child_impulse: Single;
     volume: Single;
     mass: Single;
     damage_per_fire: Single; //standardgemäß ungenutzt
     power_usage: Single;
     refire_delay: Single;
     muzzle_velocity: Single;
     toughness: Single;
     flash_particle_name: string;
     flash_radius: Single;
     light_anim: string;
     projectile_archetype: string;
     separation_explosion: string;
     auto_turret: Boolean;
     turn_rate: Single;
     lootable: Boolean;
     dry_fire_sound: string;
     dispersion_angle: single;
  end;

  RMotor = record      // nur Rakete
     nickname: string;
     lifetime: single;
     accel: single;
     delay: single;
  end;

  RMunition = record
     idname: Cardinal;
     idinfo: Cardinal;             // nur Uses Ammo True
     nickname: string;
     LODranges: string;            // nur Rakete
     DA_archetype: string;         // nur Rakete
     material_library: string;     // nur Rakete
     volume: Single;
     mass: Single;
     hit_pts: Single;
     explosion_resistance: Single; // nur Rakete
     explosion_arch: string;
     loot_appearance: string;      // nur Uses Ammo True
     units_per_container: integer; // nur Uses Ammo True
     hp_type: byte;
     cruise_disruptor: Boolean;
     requires_ammo: Boolean;
     one_shot_sound: string;
     detonation_dist: Single;      // nur mit Explosion Sinnvoll
     lifetime: Single;
     Motor: string;                // nur Rakete
     force_gun_ori: Boolean;       // eigentlich immer False
     const_effect: string;         // nur Rakete
     const_effect_delay: single;   // nur Rakete
     munition_hit_effect: string;  // nur Rakete
     weapon_type: string;
     HP_trail_parent: string;      // nur Rakete
     seeker: Byte;                 // nur Rakete
     time_to_lock: Single;         // nur Rakete
     seeker_range: Single;         // nur Rakete
     seeker_fov_deg: Single;       // nur Rakete
     max_angular_velocity: Single; // nur Rakete
     hull_damage: Single;          // nur Gun
     energy_damage: Single;        // nur Gun
  end;

  RExplosion = record
     nickname: string;
     lifetime: Single;
     process: string;                      // hier nur Disappear
     //num_child_pieces: integer;           // nur mit Simples
     //debris_impulse: single;             // nur mit Simples
     //debris_type: string;                // nur mit Simples
     //innards_debris_start_time: single;  // nur mit Simples
     //innards_debris_num: integer;        // nur mit Simples
     //innards_debris_object: string;      // nur mit Simples
     effect: string;
     radius: single;
     hull_damage: single;
     energy_damage: single;
     strength: single;
     impulse: single;                     // betrifft nur NPCs?
  end;

  RGood = record
     //nickname: string;
     //equipment: string;    // selbes wie Nickname
     //shop_archetype: string;
     //material_library: string;
     item_icon: string;
     //category: string;
     price: Cardinal;
     //combinable: Boolean;
  end;

  RGunModel = record
     model_path: string;
     material_path: string;
     envmap_path: string;
     LODRanges: string;
     HP_child: string;
     imageIndex: integer;
  end;

  RMunitionModel = record
     model_path: string;
     material_path: string;
     LODRanges: string;
     HP_trail_parent: string;
     imageIndex: integer;
  end;

  RLists = record
     effect: TStrings;
     sound: TStrings;
     LODRanges: string;
     HP_trail_parent: string;
  end;

  RSound = record
     name: string;
     path: string;
  end;

  var
    Gun: RGun;
    Motor: RMotor;
    Munition: RMunition;
    Explosion: RExplosion;
    Good: RGood;
    isGun: Boolean=True;
    energydmgfactor: single=0.5;
    GunModels: array of RGunModel;
    MunitionModels: array of RMunitionModel;
    currentGunModel: Cardinal=0;
    IconsList: array of TJPEGImage;
    Sounds: array of RSound;
    NormalShotSndIndex: array of Cardinal;
    WeaponList: array of RSound;
    StartLine, EndLine: Cardinal;

  procedure WriteNormal;
  procedure WriteGun;
  procedure WriteMunition;
  procedure WriteMotor;
  procedure WriteExplosion;
  function ErrorCheck: Boolean;
  function WarningCheck: Boolean;
  procedure SaveEquip;
  procedure SaveGood;
  function NicknameCorrection(input :string): string;
  function AddIconToList(const Path: string):integer;
  procedure AddGunModels;
  procedure AddMunitionModels;
  procedure EmptyGun;
  procedure EmptyMotor;
  procedure EmptyMunition;
  procedure EmptyExplosion;
  procedure EmptyGood;

implementation
  uses Unit1, math, uniterror, BIniReader;

  function AddIconToList(const Path: string):integer;
  begin
    if FileExists(Path) then
    begin
      SetLength(IconsList,Length(IconsList)+1);
      IconsList[High(IconsList)]:=TJPEGImage.Create;
      IconsList[High(IconsList)].LoadFromFile(Path);
      Result:=High(IconsList);
    end
    else
      Result:=-1;
  end;

  procedure FreeIconList;
  var i: integer;
  begin
    for i:=0 to High(IconsList) do
      IconsList[i].Free;
    SetLength(IconsList,0);
  end;

  procedure EmptyGun;
  begin
    with Gun do
    begin
      idname:=0;
      idinfo:=0;
      nickname:='';
      LODranges:='';
      DA_archetype:='';
      material_library:='';
      envmap_material:='';
      hp_gun_type:=0;
      HP_child:='';
      hit_pts:=4000;
      explosion_resistance:=1;
      explosion_arch:='';
      debris_type:='';
      parent_impulse:=20;
      child_impulse:=80;
      volume:=0;
      mass:=10;
      damage_per_fire:=0;
      power_usage:=0;
      refire_delay:=0;
      muzzle_velocity:=0;
      toughness:=0;
      flash_particle_name:='';
      flash_radius:=15;
      light_anim:='';
      projectile_archetype:='';
      separation_explosion:='';
      auto_turret:=False;
      turn_rate:=90;
      lootable:=False;
      dry_fire_sound:='';
      dispersion_angle:=0;
    end;
  end;

  procedure EmptyMotor;
  begin
    with Motor do
    begin
      nickname:='';
      lifetime:=2.5;
      accel:=0;
      delay:=0;
    end;
  end;

  procedure EmptyMunition;
  begin
    with Munition do
    begin
      idname:=0;
      idinfo:=0;
      nickname:='';
      LODranges:='';
      DA_archetype:='';
      material_library:='';
      volume:=0;
      mass:=1;
      hit_pts:=2;
      explosion_resistance:=1;
      explosion_arch:='';
      loot_appearance:='';
      units_per_container:=10;
      hp_type:=0;
      cruise_disruptor:=False;
      requires_ammo:=False;
      one_shot_sound:='';
      detonation_dist:=4;
      lifetime:=0;
      Motor:='';
      force_gun_ori:=False;
      const_effect:='';
      const_effect_delay:=0;
      munition_hit_effect:='';
      weapon_type:='<Nothing>';
      HP_trail_parent:='';
      seeker:=0;
      time_to_lock:=0;
      seeker_range:=0;
      seeker_fov_deg:=0;
      max_angular_velocity:=0;
      hull_damage:=0;
      energy_damage:=0;
    end;
  end;

  procedure EmptyExplosion;
  begin
    with Explosion do
    begin
      nickname:='';
      lifetime:=0;
      process:='disappear';
      //num_child_pieces:=0;
      //debris_impulse:=0;
      //debris_type:='';
      //innards_debris_start_time:=0;
      //innards_debris_num:=0;
      //innards_debris_object:='';
      effect:='';
      radius:=0;
      hull_damage:=0;
      energy_damage:=0;
      strength:=0;
      impulse:=0;
    end;
  end;

  procedure EmptyGood;
  begin
    with Good do
    begin
      //nickname:='';
      //equipment:='';
      //shop_archetype:='';
      //material_library:='';
      item_icon:='';
      //category:='';
      price:=0;
      //combinable:=False;
    end;
  end;


  procedure AddGunModels;inline;
  begin
    SetLength(GunModels, Length(GunModels)+1);
    GunModels[High(GunModels)].model_path:='';
    GunModels[High(GunModels)].material_path:='';
    GunModels[High(GunModels)].envmap_path:='';
    GunModels[High(GunModels)].LODRanges:='';
    GunModels[High(GunModels)].HP_child:='';
    GunModels[High(GunModels)].imageIndex:=-1;
  end;

  procedure AddMunitionModels;inline;
  begin
    SetLength(MunitionModels, Length(MunitionModels)+1);
    MunitionModels[High(MunitionModels)].model_path:='';
    MunitionModels[High(MunitionModels)].material_path:='';
    MunitionModels[High(MunitionModels)].LODRanges:='';
    MunitionModels[High(MunitionModels)].HP_trail_parent:='';
  end;

  procedure InitRecords;
  begin
    EmptyGun;
    EmptyMotor;
    EmptyMunition;
    EmptyExplosion;
    EmptyGood;
    energydmgfactor:=0.5;
  end;

  procedure WriteNormal;
  var i: integer;
  begin
    with Form1 do
    begin
      with Gun do
      begin
        ins_idnamegun.Value:=idname;
        ins_idinfogun.Value:=idinfo;
        ins_nickname.Text:=nickname;
        i:=ins_archetype.Items.IndexOf(ExtractFileName(DA_archetype));
        if (i=-1) then
          ins_archetype.Text:=ExtractFileName(DA_archetype)
        else
          ins_archetype.ItemIndex:=i;
        if Length(GunModels)<>0 then
          DrawPictureFromList(img_archetype,GunModels[currentGunModel].imageIndex);

        ins_weapbase.ItemIndex:=ceil(Gun.hp_gun_type * (1/11));
        ins_weapclass.ItemIndex:=Gun.hp_gun_type-(ins_weapbase.ItemIndex*11-10);
        ins_weapclass_torp.ItemIndex:=Gun.hp_gun_type-(ins_weapbase.ItemIndex*11-10);
        ins_weapbaseChange(pc_main);

        ins_firerate.Value:=refire_delay;
        ins_powerusage.Value:=power_usage;
        if isGun then
          ins_speed.Value:=muzzle_velocity
        else
          ins_speed.Value:=muzzle_velocity+(Motor.accel*Motor.lifetime);
      end;
        if isGun then
          ins_range.Value:=Munition.lifetime*Gun.muzzle_velocity
        else
          ins_range.Value:=Munition.lifetime*(Gun.muzzle_velocity+Motor.accel*Motor.lifetime);

      with Munition do
      begin
        ins_weaptype.Text:=weapon_type;
        ins_sound.Text:=one_shot_sound;
        ins_projeffect.Text:=const_effect;
        ins_cruisedisrupt.Checked:=cruise_disruptor;
        ins_requammo.Checked:=requires_ammo;
        ins_idnameammo.Value:=idname;
        ins_idinfoammo.Value:=idinfo;
        ins_seekerrange.Value:=seeker_range;
        ins_seekerfov.Value:=seeker_fov_deg;
        ins_maxangvel.Value:=max_angular_velocity;
      end;

      ins_explradius.Value:=Explosion.radius;
      ins_price.Value:=Good.price;

      if isGun then
        begin
          ins_hdmg.Value:=Munition.hull_damage;
          ins_sdmg.Value:=Munition.energy_damage+(Munition.hull_damage*energydmgfactor);
          ins_hiteffect.Text:=Munition.munition_hit_effect
        end
      else
        begin
          ins_hdmg.Value:=Explosion.hull_damage;
          ins_sdmg.Value:=Explosion.energy_damage+(Explosion.hull_damage*energydmgfactor);
          ins_hiteffect.Text:=Explosion.effect;
        end;
    end;
  end;

  procedure WriteGun;
  var i: integer;
  begin
    with Form1, Gun do
    begin
      ins_gun_idname.Value:=idname;
      ins_gun_idinfo.Value:=idinfo;
      ins_gun_nickname.Text:=nickname;
      i:=ins_gun_archetype.Items.IndexOf(ExtractFileName(DA_archetype));
      if (i=-1) then
        ins_gun_archetype.Text:=ExtractFileName(DA_archetype)
      else
        ins_gun_archetype.ItemIndex:=i;
      DrawPictureFromList(img_gun_archetype,GunModels[currentGunModel].imageIndex);
      ins_gun_guntype.ItemIndex:=hp_gun_type;
      ins_gun_firerate.Value:=refire_delay;
      ins_gun_flash_name.Text:=flash_particle_name;
      ins_gun_flash_radius.Value:=flash_radius;
      ins_gun_lightanim.Text:=light_anim;
      ins_gun_dryfiresound.Text:=dry_fire_sound;
      ins_gun_dispersangle.Value:=dispersion_angle;
      ins_gun_muzzlevel.Value:=muzzle_velocity;
      ins_gun_power.Value:=power_usage;
      ins_gun_autoturret.Checked:=auto_turret;
      ins_gun_lootable.Checked:=lootable;
      ins_gun_turnrate.Value:=turn_rate;
      ins_gun_volume.Value:=volume;
      ins_gun_mass.Value:=mass;
      ins_gun_toughness.Value:=toughness;
      ins_gun_hitpts.Value:=hit_pts;
      ins_gun_explresist.Value:=explosion_resistance;
      ins_gun_debris_type.Text:=debris_type;
      ins_gun_separtexpl.Text:=separation_explosion;
      ins_gun_parent_impulse.Value:=parent_impulse;
      ins_gun_child_impulse.Value:=child_impulse;
    end;
  end;

  procedure WriteMunition;
  var i: integer;
  begin
    with Form1, Munition do
    begin
      ins_mun_idname.Value:=idname;
      ins_mun_idinfo.Value:=idinfo;
      i:=ins_mun_archetype.Items.IndexOf(ExtractFileName(DA_archetype));
      if (i=-1) then
        ins_mun_archetype.Text:=ExtractFileName(DA_archetype)
      else
        ins_mun_archetype.ItemIndex:=i;
      if ins_mun_archetype.ItemIndex>-1 then
        DrawPictureFromList(img_mun_archetype,MunitionModels[ins_mun_archetype.ItemIndex].imageIndex);
      ins_mun_hp_type.ItemIndex:=hp_type;
      ins_mun_sound.Text:=one_shot_sound;
      ins_mun_lifetime.Value:=lifetime;
      ins_mun_detdist.Value:=detonation_dist;
      ins_mun_weaptype.Text:=weapon_type;
      ins_mun_hiteffect.Text:=munition_hit_effect;
      ins_mun_hdmg.Value:=hull_damage;
      ins_mun_sdmg.Value:=energy_damage;
      ins_mun_cruisedisrupt.Checked:=cruise_disruptor;
      ins_mun_consteffect.Text:=const_effect;
      ins_mun_consteffdelay.Value:=const_effect_delay;
      ins_mun_hp_type.ItemIndex:=seeker;
      ins_mun_timelock.Value:=time_to_lock;
      ins_mun_seekrange.Value:=seeker_range;
      ins_mun_seekfov.Value:=seeker_fov_deg;
      ins_mun_maxangvel.Value:=max_angular_velocity;
      ins_mun_reqammo.Checked:=requires_ammo;
      ins_mun_lootmodel.Text:=loot_appearance;
      ins_mun_unitpcont.Value:=units_per_container;
      ins_mun_volume.Value:=volume;
      ins_mun_mass.Value:=mass;
      ins_mun_hitpts.Value:=hit_pts;
      ins_mun_explresist.Value:=explosion_resistance;
      MunDmgSecUpdate;
    end;
  end;

  procedure WriteMotor;
  begin
    with Form1, Motor do
    begin
      ins_mot_acc.Value:=accel;
      ins_mot_lifetime.Value:=lifetime;
      ins_mot_delay.Value:=delay;
      MotorUpdateSpdLbl;
    end;
  end;

  procedure WriteExplosion;
  begin
    with Form1, Explosion do
    begin
      ins_exp_effect.Text:=effect;
      ins_exp_hdmg.Value:=hull_damage;
      ins_exp_sdmg.Value:=energy_damage;
      ins_exp_radius.Value:=radius;
      ins_exp_strength.Value:=strength;
      ins_exp_impulse.Value:=impulse;
    end;
  end;

  function NicknameCorrection(input :string): string;
  var
    i: integer;
    s: string='';
  begin
    input:=StringReplace(input,' ','_',[rfReplaceAll, rfIgnoreCase]);
    for i:=1 to Length(input) do
      if  ((input[i]='_') or
           (input[i]='#') or
          ((input[i]>='A') and (input[i]<='Z')) or
          ((input[i]>='a') and (input[i]<='z')) or
          ((input[i]>='0') and (input[i]<='9')))
      then
        s:=s+input[i];
    result:=s;
  end;

  function ErrorCheck: Boolean;
  begin
    with Form2 do
    begin
      with Gun do
      begin
        if (nickname='') then
          syn_error.Lines.Append('ERROR - Gun Internal Name not given!');
        if (GunModels[currentGunModel].model_path='') then
          syn_error.Lines.Append('ERROR - Gun Model not given!');
        if (GunModels[currentGunModel].HP_child='') then
          syn_error.Lines.Append('ERROR - Gun HP_child not given!');
        if (debris_type='') then
          syn_error.Lines.Append('ERROR - Gun Debris Type not given!');
        if (projectile_archetype='') then
          syn_error.Lines.Append('ERROR - Gun Munition Internal Name not given!');
        if (separation_explosion='') then
          syn_error.Lines.Append('ERROR - Gun Seperation_Explosion not given!');
      end;

      with Munition do
      begin
        if (nickname='') then
          syn_error.Lines.Append('ERROR - Munition Internal Name not given!');

        if (not isGun) then
        begin
          if (MunitionModels[Form1.ins_mun_archetype.ItemIndex].model_path='') then
            syn_error.Lines.Append('ERROR - Munition Model not given!');
          if (explosion_arch='') then
            syn_error.Lines.Append('ERROR - Munition Explosion Internal Name not given!');
          if (Motor='') then
            syn_error.Lines.Append('ERROR - Munition Motor not given!');
          if (MunitionModels[Form1.ins_mun_archetype.ItemIndex].HP_trail_parent='') then
            syn_error.Lines.Append('ERROR - Munition HP_trail_parent not given!');
        end;

        if (requires_ammo and (loot_appearance='')) then
          syn_error.Lines.Append('ERROR - Munition Container Model not given!');

        if (isGun and ((Form1.ins_exp_gunexp.Checked) and (explosion_arch='')) ) then
          syn_error.Lines.Append('ERROR - Gun uses Explosion, but no Explosion Internal Name given!');
      end;

      if ((not isGun) and (Motor.nickname='')) then
        syn_error.Lines.Append('ERROR - Motor Internal Name not given!');

      if ((not isGun) or ((isGun) and (Form1.ins_exp_gunexp.Checked))) then
        with Explosion do
        begin
          if (nickname='') then
            syn_error.Lines.Append('ERROR - Explosion Internal Name not given!');
          if (process='') then
            syn_error.Lines.Append('ERROR - Explosion Process not given!');
          if (effect='') then
            syn_error.Lines.Append('ERROR - Explosion Effect not given!');
        end;

      if (syn_error.Lines.Count>0) then
        ErrorCheck:=True
      else
        ErrorCheck:=False;
    end;
  end;

  function WarningCheck: Boolean;
  begin
    with Form2 do
    begin
      with Gun do
      begin
        if (GunModels[currentGunModel].material_path='') then
          syn_error.Lines.Append('WARNING - Gun Model Material not given.');
        if (hit_pts=0) then
          syn_error.Lines.Append('WARNING - Gun Hitpoints is 0.');
        if (refire_delay=0) then
          syn_error.Lines.Append('WARNING - Gun Fire Rate is 0.');
        if (refire_delay=0) then
          syn_error.Lines.Append('WARNING - Gun Speed is 0.');
        if ((flash_particle_name<>'') and (flash_radius=0)) then
          syn_error.Lines.Append('WARNING - Gun Shot Flash Radius is 0 while Effect is used.');
        if (turn_rate=0) then
          syn_error.Lines.Append('WARNING - Gun Turn Rate is 0.');
      end;

      with Munition do
      begin
        if (not isGun) then
        begin
          if (MunitionModels[Form1.ins_mun_archetype.ItemIndex].model_path='') then
            syn_error.Lines.Append('WARNING - Munition Model Material not given.');
          if (hit_pts=0) then
            syn_error.Lines.Append('WARNING - Munition Hitpoints is 0.');
          if (detonation_dist=0) then
            syn_error.Lines.Append('WARNING - Munition Detonation Distance is 0.');

          if (seeker=0) then
          begin
            if (seeker_range=0) then
              syn_error.Lines.Append('WARNING - Munition Seeker Range is 0.');
            if (seeker_fov_deg=0) then
              syn_error.Lines.Append('WARNING - Munition Seeker Field Of View is 0.');
            if (max_angular_velocity=0) then
              syn_error.Lines.Append('WARNING - Munition Rotation Speed is 0.');
          end;

        end;

        if ((requires_ammo) and (units_per_container=0)) then
          syn_error.Lines.Append('WARNING - Munition Units Per Container is 0.');
        if (lifetime=0) then
          syn_error.Lines.Append('WARNING - Munition Shot Lifetime is 0.');

        if isGun then
        begin
          if (hull_damage=0) then
            syn_error.Lines.Append('WARNING - Munition Hull Damage is 0.');
          if (energy_damage=0) then
            syn_error.Lines.Append('WARNING - Munition Shield Damage is 0.');
        end;
      end;

      if (not isGun) then
        with Motor do
        begin
          if (lifetime=0) then
            syn_error.Lines.Append('WARNING - Motor Acceleration Duration is 0.');
          if (accel=0) then
            syn_error.Lines.Append('WARNING - Motor Acceleration is 0.');
        end;

      if ((not isGun) or ((isGun) and (Form1.ins_exp_gunexp.Checked))) then
        with Explosion do
        begin
          if (radius=0) then
            syn_error.Lines.Append('WARNING - Explosion Radius is 0.');
          if (hull_damage=0) then
            syn_error.Lines.Append('WARNING - Explosion Hull Damage is 0.');
          if (energy_damage=0) then
            syn_error.Lines.Append('WARNING - Explosion Shield Damage is 0.');
        end;

      if (syn_error.Lines.Count>0) then
        WarningCheck:=True
      else
        WarningCheck:=False;
    end;
  end;

  function round6(x: Double):Double;
  begin
    round6:=round(1000000*x)/1000000;
  end;

  procedure SaveEquip;
  var  inifile: TStrings;
  begin
    with Form1 do
    begin
      if FileExists(save_equip.FileName) then
        inifile:=BIniReader.LoadBiniFile(save_equip.FileName)
      else
        inifile:=TStringList.Create;

      inifile.Append('');
      //inifile.Append(';------Weapon: '+Gun.nickname);
      if (not isGun or (isGun and ins_exp_gunexp.Checked)) then
        with Explosion do
        begin
          inifile.Append('[Explosion]');
          inifile.Append('nickname = '+nickname);
          inifile.Append('lifetime = '+FloatToStr(round6(lifetime)));
          inifile.Append('process = '+process);
          //inifile.Append('num_child_pieces = '+IntToStr(num_child_pieces));
          //inifile.Append('debris_impulse = '+FloatToStr(round6(debris_impulse)));
          //inifile.Append('debris_type = '+debris_type);
          //inifile.AdAppendd('innards_debris_start_time = '+FloatToStr(round6(innards_debris_start_time)));
          //inifile.Append('innards_debris_num = '+IntToStr(innards_debris_num));
          //inifile.Append('innards_debris_object = '+innards_debris_object);
          inifile.Append('effect = '+effect);
          inifile.Append('radius = '+FloatToStr(round6(radius)));
          inifile.Append('hull_damage = '+FloatToStr(round6(hull_damage)));
          inifile.Append('energy_damage = '+FloatToStr(round6(energy_damage)));
          inifile.Append('strength = '+FloatToStr(round6(strength)));
          inifile.Append('');
        end;

      if (not isGun) then
        with Motor do
        begin
          inifile.Append('[Motor]');
          inifile.Append('nickname = '+nickname);
          inifile.Append('lifetime = '+FloatToStr(round6(lifetime)));
          inifile.Append('accel = '+FloatToStr(round6(accel)));
          inifile.Append('delay = '+FloatToStr(round6(delay)));
          inifile.Append('');
        end;

      with Munition do
      begin
        inifile.Append('[Munition]');
        inifile.Append('nickname = '+nickname);

        if (not isGun or (isGun and requires_ammo)) then
          inifile.Append('ids_name = '+IntToStr(idname));

        if requires_ammo then
        begin
          inifile.Append('ids_info = '+IntToStr(idinfo));
          inifile.Append('loot_appearance = '+loot_appearance);
          inifile.Append('units_per_container = '+IntToStr(units_per_container));
        end;

        if (not isGun) then
        begin
          inifile.Append('DA_archetype = '+DA_archetype);
          inifile.Append('material_library = '+material_library);
          inifile.Append('LODranges = '+LODranges);
          inifile.Append('HP_trail_parent = '+HP_trail_parent);
          inifile.Append('Motor = '+Motor);
          case seeker of
              0: inifile.Append('seeker = LOCK');
              1: inifile.Append('seeker = DUMB');
          end;
          inifile.Append('time_to_lock = '+FloatToStr(round6(time_to_lock)));
          inifile.Append('seeker_range = '+FloatToStr(round6(seeker_range)));
          inifile.Append('seeker_fov_deg = '+FloatToStr(round6(seeker_fov_deg)));
          inifile.Append('max_angular_velocity = '+FloatToStr(round6(max_angular_velocity)));
        end;

        if isGun then
        begin
          inifile.Append('hp_type = hp_gun');
          inifile.Append('hull_damage = '+FloatToStr(round6(hull_damage)));
          inifile.Append('energy_damage = '+FloatToStr(round6(energy_damage)));
          inifile.Append('munition_hit_effect = '+munition_hit_effect);
          if (LowerCase(weapon_type)<>'<nothing>') then
            inifile.Append('weapon_type = '+weapon_type);
        end
        else
          case hp_type of
              0: inifile.Append('hp_type = hp_gun');
              1: inifile.Append('hp_type = hp_torpedo');
          end;

        if (not isGun or (isGun and ins_exp_gunexp.Checked)) then
          inifile.Append('explosion_arch = '+explosion_arch);

        inifile.Append('volume = '+FloatToStr(round6(volume)));
        inifile.Append('mass = '+FloatToStr(round6(mass)));
        inifile.Append('hit_pts = '+FloatToStr(round6(hit_pts)));
        inifile.Append('explosion_resistance = '+FloatToStr(round6(explosion_resistance)));
        inifile.Append('cruise_disruptor = '+BoolToStr(cruise_disruptor, True));
        inifile.Append('requires_ammo = '+BoolToStr(requires_ammo, True));
        inifile.Append('one_shot_sound = '+one_shot_sound);
        inifile.Append('lifetime = '+FloatToStr(round6(lifetime)));
        inifile.Append('force_gun_ori = '+BoolToStr(force_gun_ori, True));
        inifile.Append('const_effect = '+const_effect);
        inifile.Append('const_effect_delay = '+FloatToStr(round6(const_effect_delay)));
        inifile.Append('');
      end;

      with Gun do
      begin
        inifile.Append('[Gun]');
        inifile.Append('nickname = '+nickname);
        inifile.Append('ids_name = '+IntToStr(idname));
        inifile.Append('ids_info = '+IntToStr(idinfo));
        inifile.Append('DA_archetype = '+DA_archetype);
        inifile.Append('material_library = '+material_library);
        if GunModels[currentGunModel].envmap_path<>'' then
          inifile.Append('envmap_material = '+envmap_material);
        inifile.Append('LODranges = '+LODranges);
        inifile.Append('HP_child = '+HP_child);
        inifile.Append('volume = '+FloatToStr(round6(volume)));
        inifile.Append('mass = '+FloatToStr(round6(mass)));
        inifile.Append('hit_pts = '+FloatToStr(round6(hit_pts)));
        inifile.Append('explosion_resistance = '+FloatToStr(round6(explosion_resistance)));
        if explosion_arch<>'' then
          inifile.Append('explosion_arch = '+explosion_arch);
        inifile.Append('debris_type = '+debris_type);
        inifile.Append('separation_explosion = '+separation_explosion);
        inifile.Append('parent_impulse = '+FloatToStr(round6(parent_impulse)));
        inifile.Append('child_impulse = '+FloatToStr(round6(child_impulse)));
        if (hp_gun_type>0)then
          inifile.Append('hp_gun_type = '+ins_gun_guntype.Items.Strings[hp_gun_type]);
        inifile.Append('damage_per_fire = '+FloatToStr(round6(damage_per_fire)));
        inifile.Append('power_usage = '+FloatToStr(round6(power_usage)));
        if (refire_delay=0) then
          inifile.Append('refire_delay = 0')
        else
          inifile.Append('refire_delay = '+FloatToStr(round6(1/refire_delay)));
        inifile.Append('muzzle_velocity = '+FloatToStr(round6(muzzle_velocity)));
        inifile.Append('toughness = '+FloatToStr(round6(toughness)));
        inifile.Append('projectile_archetype = '+projectile_archetype);
        inifile.Append('flash_particle_name = '+flash_particle_name);
        inifile.Append('flash_radius = '+FloatToStr(round6(flash_radius)));
        inifile.Append('light_anim = '+light_anim);
        if Munition.requires_ammo then
          inifile.Append('dry_fire_sound = '+dry_fire_sound);
        inifile.Append('auto_turret = '+BoolToStr(auto_turret, True));
        inifile.Append('turn_rate = '+FloatToStr(round6(turn_rate)));
        inifile.Append('dispersion_angle = '+FloatToStr(round6(dispersion_angle)));
        inifile.Append('lootable = '+BoolToStr(lootable, True));
      end;
      inifile.SaveToFile(save_equip.FileName);
    end;
    inifile.Free;
  end;

  procedure SaveGood;
  var  inifile: TStrings;
  begin
    with Form1 do
    begin
      if FileExists(save_good.FileName) then
        inifile:=BIniReader.LoadBiniFile(save_good.FileName)
      else
        inifile:=TStringList.Create;

      inifile.Append('');
      inifile.Append(';------Weapon: '+Gun.nickname);
      with Good, Gun do
      begin
        inifile.Append('[Good]');
        inifile.Append('ids_name = '+IntToStr(idname)+' ;not used');
        inifile.Append('nickname = '+nickname);
        inifile.Append('equipment = '+nickname);
        inifile.Append('shop_archetype = '+GunModels[currentGunModel].model_path);
        inifile.Append('material_library = '+GunModels[currentGunModel].material_path);
        //inifile.Append('item_icon = '+item_icon);
        if radio_weapontype.ItemIndex=0 then
          if Gun.hp_gun_type>11 then
            inifile.Append('item_icon = equipment\models\commodities\nn_icons\EQUIPICON_turret.3db')
          else
            inifile.Append('item_icon = equipment\models\commodities\nn_icons\EQUIPICON_gun.3db')
        else
          if Munition.hp_type=1 then
            inifile.Append('item_icon = equipment\models\commodities\nn_icons\EQUIPICON_torplauncher.3db')
          else
            inifile.Append('item_icon = equipment\models\commodities\nn_icons\EQUIPICON_missilelauncher.3db');
        inifile.Append('category = equipment');
        inifile.Append('price = '+IntToStr(price));
        inifile.Append('combinable = False');
      end;

      if Munition.requires_ammo then
        with Good, Munition do
        begin
          inifile.Append('');
          inifile.Append('[Good]');
          inifile.Append('ids_name = '+IntToStr(idname)+' ;not used');
          inifile.Append('nickname = '+nickname);
          inifile.Append('equipment = '+nickname);
          inifile.Append('shop_archetype = '+MunitionModels[ins_mun_archetype.ItemIndex].model_path);
          inifile.Append('material_library = '+MunitionModels[ins_mun_archetype.ItemIndex].material_path);
          //inifile.Append('item_icon = '+item_icon);
          if Munition.hp_type=1 then
            inifile.Append('item_icon = equipment\models\commodities\nn_icons\EQUIPICON_torplauncher.3db')
          else
            inifile.Append('item_icon = quipment\models\commodities\nn_icons\EQUIPICON_missiles.3db');
          inifile.Append('category = equipment');
          inifile.Append('price = '+IntToStr(price div 20));
          inifile.Append('combinable = True');
          inifile.Append('free_ammo = '+nickname+', 10');
        end;
      inifile.SaveToFile(save_good.FileName);
    end;
    inifile.Free;
  end;

initialization
  InitRecords;

finalization
  SetLength(GunModels,0);
  SetLength(MunitionModels,0);
  SetLength(Sounds,0);
  SetLength(WeaponList,0);
  SetLength(NormalShotSndIndex,0);
  FreeIconList;

end.

